

/* Revision History
 *
 * $Id: perlutils.cc,v 1.5 2001/04/09 15:36:37 ulammers Exp $
 *
 */

#include <ostream.h>
#include <cerrno>

// Under some conditions, the Perl headers are not protected against
// name mangling. Make sure they are.
extern "C" {
#include "EXTERN.h"
#include <cassert>
#include <cstring>
#include <perl.h>
}

#include "perlutils.h"


typedef struct {
    char type;       
    void *pdata;
} Out_Param;


int
perl_eval_va (const char *str, ...)
{
    /* Evals a string, returns -1 if unsuccessful, else returns
     *  the number of return params
     *  char buf[10]; int a;
     *  perl_eval_va ("$a = 10; ($a, $a+1)",
     *                "i", &a,
     *                "s", buf,
     *                 NULL);
     */
       
    SV*       sv     = newSVpv((char *)str,0);
    va_list   vl;
    char      *p     = NULL;  
    int       i      = 0; 
    int       nret   = 0;     /* number of return params expected*/
    int       result = 0;
    Out_Param op[20];
    int ii; double d;

    djSP; /* GV -- this used to be dSP, but that fails on Linux because of a problem with errno and threads */
    ENTER;
    SAVETMPS;
    PUSHMARK(sp);
    va_start (vl, str);

    while ((p = va_arg(vl, char *))) {
        if ((*p != 's') && (*p != 'i') && (*p != 'd')) {
            fprintf (stderr, "perl_eval_va: Unknown option \'%c\'.\n"
                              "Did you forget a trailing NULL ?\n", *p);
            return -1;
        }
        op[nret].pdata = (void*) va_arg(vl, char *);
        op[nret++].type = *p;
    }
    va_end(vl);
    PUTBACK;
    result = perl_eval_sv(sv, (nret == 0) ? G_DISCARD :
                              (nret == 1) ? G_SCALAR  :
                                            G_ARRAY  );

    SPAGAIN;
    if (SvTRUE(GvSV(errgv))) { /* errgv == $@ */
        fprintf (stderr, "Eval error: %s", SvPV(GvSV(errgv), na)) ;
        return -1;
    }
    SvREFCNT_dec(sv);
    /*printf ("nret: %d, result: %d\n", nret, result);*/
    if (nret > result)
        nret = result;

    for (i = --nret; i >= 0; i--) {
        switch (op[i].type) {
        case 's':
            str = POPp;
            /*printf ("String: %s\n", str);*/
            strcpy((char *)op[i].pdata, str);
            break;
        case 'i':
            ii = POPi;
            /*printf ("Int: %d\n", ii);*/
            *((int *)(op[i].pdata)) = ii;
            break;
        case 'd':
            d = POPn;
            /*printf ("Double: %f\n", d);*/
            *((double *) (op[i].pdata)) = d;
            break;
        }
   }
   FREETMPS ;
   LEAVE;
   return result;
}    



// #ifdef __cplusplus
// extern "C" {
// #endif

// #include <EXTERN.h>
// #include <perl.h>

// #ifdef __cplusplus
// }
// #  ifndef EXTERN_C
// #    define EXTERN_C extern "C"
// #  endif
// #else
// #  ifndef EXTERN_C
// #    define EXTERN_C extern
// #  endif
// #endif

#ifndef EXTERN_C
#  define EXTERN_C extern "C"
#endif

EXTERN_C void xs_init _((void));

EXTERN_C void boot_DynaLoader _((CV* cv));

EXTERN_C void
xs_init()
{
	char *file = __FILE__;
	dXSUB_SYS;

	/* DynaLoader is a special case */
	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
}


void
initialisePerl( const string & perllib )

{
	static char *			argv[] = { "", (char*)perllib.c_str() };

	static PerlInterpreter *	perlInterpreter;

	perlInterpreter = perl_alloc();
	perl_construct( perlInterpreter );
	perl_parse( perlInterpreter, xs_init, 2, argv, 0 );

}


int
evalperl( const char * expression )
{

	int				status;

	status = perl_eval_va( (char *)expression, 0 );

	return status;

}


