/***************************************************************************

   QuotedString.h                                               SAS

   (c) 1998 European Space Agency - Written by Marco Beijersbergen

   $Id: QuotedString.h,v 1.3 2001/04/09 15:36:38 ulammers Exp $

 **************************************************************************/

#ifndef QuotedString_h
#define QuotedString_h

#include <string>
#include <iostream>

class Quotes {

  public:

    Quotes(char o='"', char c='"') : open(o), close(c) {}

    char open, close;
};

class QuotedString : public string {

  public:

    QuotedString() {}

    QuotedString(const Quotes& q) : _quotes(q) {}
    QuotedString(char open, char close) : _quotes(open,close) {}

    QuotedString(const string& s) : string(s) {}
    QuotedString(const string& s, size_type pos, size_type n = npos)
	 : string(s,pos,n) {}
    QuotedString(const char *t, int len) : string(t,len) {}
    QuotedString(const char *t) : string(t) {}
    QuotedString(size_type n, char c) : string(n,c) {}

    const Quotes& quotes() const { return _quotes; }
    Quotes& quotes() { return _quotes; }
    void quotes(char open, char close) { _quotes = Quotes(open,close); }
    void quotes(const Quotes& q) { _quotes = q; }

    bool operator==(const QuotedString& s) const;

  private:

    Quotes _quotes;
};

inline bool QuotedString::operator==(const QuotedString& s) const
{
  return (compare(s) == 0);
}

inline istream& operator>>(istream& is, QuotedString& qs)
{
  char ch;

  if (is >> ws && is.get(ch)) {

    if (ch == qs.quotes().open) {
      qs = "";
      while (is.get(ch) && ch != qs.quotes().close)
        qs += ch;
    }
    else {
      is.unget();
      is >> (string&)qs;

      // Workaround for string bug: operator>> appends the trailing 0
      // from a strstream.
      if (qs[qs.length()-1] == 0) qs.resize(qs.length()-1);
    }
  }
  return is;
}

inline ostream& operator<<(ostream& os, const QuotedString& s)
{
  if (s.empty() || s.find_first_of(" \t\n\r") != string::npos)
    os << s.quotes().open << (string&)s << s.quotes().close;
  else
    os << (string&)s;
  return os;
}

#endif






