//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	SimpleScanner.cc--- Implements the SimpleScanner class which represents
//                          a simple scanner.
//
//	DESCRIPTION:
//
//	USAGE EXAMPLES:
//
//	CHANGE HISTORY:
//
//	AUTHOR:
//		M.Thomas
//
//	REFERENCES:
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/utils/src/SimpleScanner.cc,v 1.3 2000/02/22 14:35:36 mthomas Exp $
///

#include <strstream>

#include "Tracer.h"
#include "SimpleScanner.h"
#include "stringutil.h"


SimpleScanner::Expected::Expected( const string& message )
	: _message( message )
{
	PROC
	ENDPROC
}


string
SimpleScanner::Expected::message() const
{
	PROC
	RETURN _message;
}


SimpleScanner::SimpleScanner(
	const string&	s,
	const string&	separators )
	:
		_source( s ),
		_ifs(*( new istrstream( _source.c_str() ))),
		_separators( separators )
{
	PROC
	fetchNextToken();
	ENDPROC
}


SimpleScanner::~SimpleScanner()
{
	PROC
	delete &_ifs;
	ENDPROC
}


bool
SimpleScanner::eof() const
{
	PROC
	//RETURN _ifs.eof();
	//RETURN _lookahead.id() == SimpleToken::Eof;
	RETURN _ifs.eof() && _lookahead.id() == SimpleToken::Eof;
}


const SimpleToken&
SimpleScanner::next() const
{
	PROC
	RETURN _lookahead;
}


SimpleToken
SimpleScanner::accept()
{
	PROC
	const SimpleToken	nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


void
SimpleScanner::skip()
{
	PROC
	fetchNextToken();
	ENDPROC
}


SimpleToken
SimpleScanner::match( SimpleToken::Id id ) throw( Expected )
{
	PROC
	if( _lookahead.id() != id )
	{
		throw Expected( string( "A token with token ID " ) + toString( int(id) ) + " (" + SimpleToken( id ).idAsString() + ") was expected" );
	} // if
	SimpleToken nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


SimpleToken
SimpleScanner::match( char c  ) throw( Expected )
{
	PROC
	if( _lookahead.id() != SimpleToken::Character || _lookahead.asChar() != c )
	{
		throw Expected( string( "The character '" ) + c + "' was expected" );
	} // if
	SimpleToken nextToken = _lookahead;
	fetchNextToken();
	RETURN nextToken;
}


bool
SimpleScanner::nextIsCharacter( char thisCharacter ) const
{
	PROC
	if(
		_lookahead.id() == SimpleToken::Character
		&&
		_lookahead.asChar() == thisCharacter )
	{
		ALTRETURNV( true );
	} // if
	RETURN false;
}


bool 
SimpleScanner::nextIsString() const
{
	PROC
	if( _lookahead.id() == SimpleToken::String )
	{
		ALTRETURNV( true );
	} // if
	RETURN false;
}


void
SimpleScanner::fetchNextToken()
{
	PROC
	if( !_ifs )
	{
		_lookahead = SimpleToken::Eof;
		ALTRETURN;
	} // if
	while( _ifs )
	{
		char	nextChar;
		_ifs >> nextChar;
		if( !_ifs )
		{
			_lookahead = SimpleToken::Eof;
		} 
		else if( _separators.find( nextChar ) == string::npos )
		{
			string	str;
			//str = str + nextChar;
			//_ifs >> nextChar;
			while( _ifs && _separators.find( nextChar ) == string::npos )
			{
				str = str + nextChar;
				_ifs >> nextChar;
			} // while
			_ifs.unget();
			_lookahead = str;
			break;
		}
		else
		{
			_lookahead = nextChar;
			break;
		} // if
	} // while
	ENDPROC
}


string
SimpleScanner::remainder() const
{
	PROC

	string	r;
	char 	c;

	switch( _lookahead.id() )
	{
		case SimpleToken::String :
			r = r + _lookahead.asString();
			break;
		case SimpleToken::Character :
			r = r + _lookahead.asChar();
			break;
		default :
			break;
	} // switch

	_ifs >> c;
	while( _ifs )
	{
		r = r + c;
	 	_ifs >> c;
	} // while
	RETURN r;
}



