//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1998 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	StopWatch--- defines the StopWatch class that represents a stop watch.
//
//	DESCRIPTION:
//
//		enum Type { CPU, Calender };
//			// Specifies the type of stop watch:
//			// CPU : The CPU clock will be tracked (in seconds).
//			// Calender : The Calender clock will be tracked (in seconds).
//
//		StopWatch( Type type = CPU, bool autoStart = true );
//			// Instantiate an StopWatch object, specifying it's type.
//			// Note that the Stop Watch will be immediately started
//			// if autoStart is true;
//
//		~StopWatch();
//
//		void stop();
//			// Stop the clock.
//
//		bool stopped() const;
//			// Returns true if the clock has been stopped.
//
//		void start();
//			// Start the clock.
//
//		void split();
//			// Not implemented
//
//		double splitElapsed() const;
//			// Not implemented
//
//		Type type() const;
//			// Returns the type of the stop watch.
//
//		double elapsed() const;
//			// Returns the elapsed time of the clock (in seconds).
//
//		void reset();
//			// Resets the elapsed time of the clock to zero.
//
//	USAGE EXAMPLES:
//
//	CHANGE HISTORY:
//		MT, 1998-10-01: 0.1.0: - first version
//
//	AUTHOR:
//		M.Thomas
//
//	REFERENCES:
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/utils/src/StopWatch.h,v 1.2 1999/04/01 08:16:53 mthomas Exp $
///
#ifndef __STOP_WATCH_H
#define __STOP_WATCH_H


class Clock;

class StopWatch
{

	public :

		enum Type { CPU, Calender };

		StopWatch( Type type = CPU, bool autoStart = true );
		~StopWatch();
		void stop();
		bool stopped() const;
		void start();
		void split();
		double splitElapsed() const;
		Type type() const;
		double elapsed() const;
		void reset();

	private :

		Clock *	_clock;

};


#endif

