/*****************************************************************************
 *
 *  stringFns.cc                                XMM Science Analysis Subsystem
 *
 *  string functions
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: StringFns.cc,v 1.2 2000/01/19 15:37:55 jbrumfit Exp $
 *
 *****************************************************************************/

#include "StringFns.h"       // Application headers


/*** Split a string into tokens, using specified separator character ***/

list<string> StringFns::splitString(const string& s, const string &separator)
{
    list<string> strlist;

    if(s.length() > 0)
    {
        string::size_type i = s.find_first_not_of(separator);
        string::size_type j = 0;
        while(j != string::npos && i != string::npos)
        {
            j = s.find_first_of(separator,i);
            strlist.push_back(s.substr(i,j-i));
            i = s.find_first_not_of(separator,j);
        }
    }
    return strlist;
}


/*
 *  Split a quoted string into tokens. Set 'ok=false' if quotes are mismatched.
 */

list<string> StringFns::splitStringList(const string& str, bool* ok)
{
    bool misMatch = false;
    list<string> strlist;
    const char* p = str.c_str();
    char c =*p++;

    while(c)
    {
        string token = "";

        // Skip leading space
	while(c && (c ==' ' || c == '\t')) 
	    c=*p++;

	// Quotes
	if(c == '"' || c == '\'') {
	    char quote = c;
	    c=*p++;
	    while(c && (c != quote)) {
	        token += c;
		c=*p++;
	    }
	    if(c != quote)
	        misMatch = true;           // Mis-matched quotes

	    c=*p++;
	    strlist.push_back(token);
	} 
	else if(c && (c != ' ' && c != '\t')) 
	{
	    // Simple token
	    while(c && (c != ' ' && c != '\t' && c != '"')) {
	        token += c;
		c=*p++;
	    }
	    strlist.push_back(token);
	}
    }
    if(ok)
        *ok = !misMatch;

    return strlist;
}


/*
 *  Split a string into tokens separated by delimiter characters.
 */

list<string> StringFns::splitDelimitedString(const string& s, const string& delim)
{
    list<string> strlist;

    if(s.length() > 0)
    {
        string::size_type i = 0;
        string::size_type j = 0;
        while(j != string::npos && i != string::npos)
        {
            j = s.find_first_of(delim,i);
            strlist.push_back(s.substr(i,j-i));
            i = j+1; 
        }
    }
    return strlist;
}


/*
 *  Parse an integer.
 *  If valid, return true and pass result via argument 'result'.
 */

bool StringFns::parseInt(const string& str, int& result)
{
    char* endptr;
    long i = strtol(str.c_str(),&endptr,10);
    bool ok = true;
    while (char c = *endptr++) 
        if(!isspace(c)) 
            ok = false;

    if(ok)
        result = i;

    return ok;
}


/*
 *  Parse a double.
 *  If valid, return true and pass result via argument 'result'.
 */

bool StringFns::parseDouble(const string& str, double& result)
{
    char* endptr;
    double d = strtod(str.c_str(),&endptr);
    bool ok = true;
    while (char c = *endptr++) 
        if(!isspace(c)) 
            ok = false;

    if(ok)
        result = d;

    return ok;
}
