// Copyright 1999 (c) European Space Agency - written by Marco Beijersbergen

#ifndef Tracer_h
#define Tracer_h

#include <string>
#include <list>
#include <iostream>

class Tracer {

  public:

    enum Verbosity { Silent=0, Stack, Sparse, Verbose };

    Tracer(ostream& os = cerr);

    ~Tracer();

    void in(const string& mess);

    void out();

    void trace(bool enabled=true) { _verbosity = (enabled ? Verbose:Sparse); }

    bool tracing() const { return _verbosity >= Verbose; }

    void verbosity(Verbosity v) { _verbosity = v; }

    Verbosity verbosity() const { return _verbosity; }

  private:

    string indent() const { return string(2*_indent,' '); }

    ostream& _os;
    Verbosity _verbosity;
    int _indent;
    list<string> _stack;
};

extern Tracer tracer;

#ifdef TRACE

#define PROC            { if (tracer.verbosity()) tracer.in(__PRETTY_FUNCTION__); }
#define ENDPROC         { if (tracer.verbosity()) tracer.out(); }
#define RETURN          return (tracer.verbosity() ? tracer.out(),0:0 ),
#define ALTRETURNV(x)   { if (tracer.verbosity()) tracer.out(); return x; }
#define ALTRETURN       { if (tracer.verbosity()) tracer.out(); return; }

#define SAS_DEBUG(command)	{ if (tracer.verbosity() >= Tracer::Sparse) { command } }

#else

#define PROC
#define ENDPROC
#define RETURN          return
#define ALTRETURNV(x)   { return (x); }
#define ALTRETURN       { return; }

#define SAS_DEBUG(command)

#endif

#endif
