//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2009 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	common.h	--- utility marcos
//
//	DESCRIPTION:
//		This acts as a pool for macro definitions whose use in application
//		code is likely to enhance the portabilty.
//
//	CHANGE HISTORY:
//		UL, 1997-11-03: 1.0.0: - initial version
//		UL, 1999-09-29: 1.1.0: - changes preprocessor symbols names
//								 which control which version of CXXFUNC
//								 is used; do not bail out if none of the
//								 listed symbols is defined but use
//								 default version
//
//	AUTHOR:
//		SAS developers
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//			
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/utils/src/common.h,v 1.5 1999/09/29 12:34:36 ulammers Exp $
//

#ifndef COMMON_H
#define COMMON_H

//
//	string concatenation
//
#define YCAT(a, b)      a ## b
#define CAT(a, b)       YCAT(a, b)
#define CCAT(a, b, c)   CAT(CAT(a, b), c)

//
//	number of array elements
//
#define NUM_OF(a)		(sizeof(a)/sizeof(a[0]))

//
//	the deprecated casts
//
#define S_CAST(a, t)	(static_cast<t>(a))
#define PTR_CAST(a, t)	(reinterpret_cast<t *>(a))
#define INT(a)			(S_CAST(a, int))
#define SHORT(a)		(S_CAST(a, short))
#define UNT(a)			(S_CAST(a, unsigned))
#define DBL(a)			(S_CAST(a, double))
#define FLT(a)			(S_CAST(a, float))
#define INTP(a)			(PTR_CAST(a, int))
#define UNSP(a)			(PTR_CAST(a, unsigned))
#define SHORTP(a)		(PTR_CAST(a, short))
#define FLOATP(a)		(PTR_CAST(a, float))
#define DOUBLEP(a)		(PTR_CAST(a, double))

//
//	name of C++/C function name which is to be called from Fortran 90
//	should be moved to f90cxx.h
// 
#if defined (SOLARIS) || defined (DECOSF) || defined(LINUX)
    #define CXXFUNC(cname, f90name)   CAT(cname, _)
#elif defined (HPUX)
    #define CXXFUNC(cname, f90name)   CAT(a, )
#else
	//	unknown - assume the SOLARIS case
    #define CXXFUNC(cname, f90name)   CAT(cname, _)
#endif

#if defined(SAS_DEBUG)
	#define DEBUGCODE(a)	a;
#else
	#define DEBUGCODE
#endif

#endif
