
#include "stringutil.h"
#include <Tracer.h>


string trim(const string& s, bool leading, const string &whitespace)
{
  PROC

  if (s.empty()) ALTRETURNV( s );
  int i;
  for (i = s.length()-1; i>=0; --i)
    if (s[i] != ' ') break;
  string r = s.substr(0,i+1);

  if (leading) {
    // Remove leading spaces. Check also for n < r.length() to work around
    // a bug in DEC/OSF-gnu-2.7.2.2, which gives a strange n in case r is empty.
    unsigned n = r.find_first_not_of(whitespace);
    if (n != string::npos && n > 0 && n < r.length())
	r = r.substr(n,string::npos);
  }

  RETURN r;
}


string
trimDigits( const string & s )
{
	PROC
 	if (s.empty()) ALTRETURNV( s );

	unsigned int i = 0;
	for( i = s.length()-1; i; --i ) if( !isdigit( s[i] )) break;
  	RETURN s.substr(0,i+1);
}


string unquote(const string& s)
{
  PROC
  string r = trim(s,true);

  if (!r.empty()) {
    if (r[0] == '\'') {
      if (r.length() <= 1 || r[r.length()-1] != '\'')
	cerr << "missing ' in " << s << endl;
      else
	ALTRETURNV( r.substr(1,r.length()-2) );
    }
    else if (r[0] == '"') {
      if (r.length() <= 1 || r[r.length()-1] != '"')
	cerr << "missing '' in " << s << endl;
      else
	ALTRETURNV( r.substr(1,r.length()-2) );
    }
  }

  // If not quoted, return original string.
  RETURN s;
}

string uppercase(const string& s)
{
  PROC
  string cap = s;
  for (unsigned i = 0; i < cap.length(); ++i)
    if (cap[i] >= 'a' && cap[i] <= 'z') cap[i] += 'A' - 'a';
  RETURN cap;  
}

string lowercase(const string& s)
{
  PROC
  string cap = s;
  for (unsigned i = 0; i < cap.length(); ++i)
    if (cap[i] >= 'A' && cap[i] <= 'Z') cap[i] -= 'A' - 'a';
  RETURN cap;  
}


vector<string> quotedBreakup(const string &s, const string &sep,
							 const string &quote)
{
	PROC

	vector<string> v;
	string::size_type left = 0, act = 0, len = s.length();
	bool inQuoted = false;
	while(act < len) {
		if (sep.find_first_of(s[act], 0) != string::npos) {
			//	found a separator
			++act;
			continue;
		}
		inQuoted = quote.find_first_of(s[act], 0) != string::npos;
		left = act;
		++act;
		//	found beginning of field - search the end
		while(act < len) {
			string::size_type p1, p2;
			if ((p1 = quote.find_first_of(s[act], 0)) != string::npos)
				if (!inQuoted) {
					//	found a quote delimiter
					//	check if it is really the start of a quoted
					//	substring by finding the matching closing
					//	delimiter; if the delimter is an open parenthesis
					//	we look for the corresponding closing one
					char closingDelim;
					switch(quote[p1]) {
						case '(': closingDelim = ')'; break;
						case '{': closingDelim = '}'; break;
						case '[': closingDelim = ']'; break;
						default: closingDelim = quote[p1]; break;
					}
					p2 = s.find_first_of(closingDelim, act+1);
					if (p2 != string::npos)
						//	yes found closing delimiter - advance actual
						//	pointer to character after that
						act = p2;
				} else
					inQuoted = false;
			else if (sep.find_first_of(s[act], 0) != string::npos && !inQuoted)
				break;
			++act;
		}
		v.push_back(s.substr(left, act-left));
		++act;
	}

	RETURN v;	
} // quotedBreakup()


vector<string> breakup(const string &str, const string &sep)
{
	PROC

	vector<string> vstr;
	string::size_type left = 0;
	while ((left = str.find_first_not_of(sep, left)) != string::npos) {
		string::size_type right = str.find_first_of(sep, left);
		if (right != string::npos)
			vstr.push_back(str.substr(left, right-left));
		else
			vstr.push_back(str.substr(left, str.length()-left));
		left = right;
	}

	RETURN vstr;
}



vector<string>
fullbreakup(const string &str, const string& sep = ":" )
{
	PROC

	vector<string> vstr;
	string::size_type left = 0;

	while( 1 )
	{
		string::size_type right = str.find_first_of( sep, left );
		if( right == string::npos )
		{
			vstr.push_back( str.substr( left, str.length() - left ));
			break;
		}
		else
		{
			vstr.push_back( str.substr( left, right - left ));
		} // if
		left = right + 1;
	} // while

	RETURN vstr;
}



string
buildup( const vector<string>& tokens, char sep = ':' )
{
	PROC

	string	str;

	if( tokens.size() > 0 )
	{
		str = tokens[ 0 ];
		for( unsigned int i = 1; i < tokens.size(); ++i )
		{
			str += sep + tokens[ i ];
		} // for
	} // if
	RETURN str;
}


string
removeStringToken(
		const string&		tokens,
		unsigned int		position = 0,
		char			sep = ':' )
{
	PROC
	string	str;
	vector<string>	listOfTokens = fullbreakup( tokens, toString( sep ));
	vector<string>::iterator i = listOfTokens.begin();
	for( unsigned int j = 0; j < position; ++j ) ++i;
	listOfTokens.erase( i );

	RETURN buildup( listOfTokens, sep );
}


string
stringToken(
		const string&		tokens,
		unsigned int		position = 0,
		char			sep = ':' )
{
	PROC

	string	token;

	if( !tokens.empty() )
	{
		vector<string> listOfTokens = fullbreakup( tokens, toString( sep ));
		token = listOfTokens[ position ];
	} // if

	RETURN token;
}



bool
isBlank(
	const string&		thisString )
{
	PROC
	RETURN
		thisString.find_first_not_of( ' ' ) == string::npos
		|| thisString.empty();
}

