// (C) 1997 European Space Agency - written by Marco Beijersbergen

#ifndef __stringutil_h
#define __stringutil_h

#include <string>
#include <Tracer.h>
#include <strstream>
#include <iomanip>
#include <vector>

string trim(const string& s, bool leading=false, const string &ws=" \n\t");

string trimDigits( const string & s );

string unquote(const string&);

string lowercase(const string&);

string uppercase(const string&);

template<class T> string toString(const T& t, int prec = -1)
{
	PROC
	strstream s;
	if (prec >= 0)
		s.precision(prec);
	s<<t<<ends;
	char *p =  s.str();
	string ss = p;
	delete p;
	RETURN ss;
}

template<class T> T toNumber(const string &s)
{
	PROC

	strstream str; str<<s<<ends;
	T v;	str>>v;

	RETURN v;
}


//
//	quotedBreakup: break a string up into tokens separated by unquoted
//				   white-space characters (along the lines of std::strtok());
//				   the delimiters can also contain open parenthesis "({["
//				   in which case the corresponding closing delimiter is
//				   taken as the associated closing parenthesis
//	breakup		 : same as quotedBreakup() but quoting characters are
//				   ignored
//
//		||			=> {}
//		|:|			=> {}
//		|"|			=> {"}
//		|s1|		=> {s1}
//		|s1"::"x|	=> {s1"::"x}
//		|s1:s2:s"3|	=> {s1,s2,s"3}
//		|s1:s2:s3|	=> {s1,s2,s3}
//		|"x::":s1|	=> {"x::",s1}
//		|x(1, 2) x={x 2}  => {x(1, 2), x, {2 3}} with quote="({"
//
//	UL, Wed Jun  3 13:57:07 MET DST 1998
//
//
vector<string> quotedBreakup(const string &str,
							 const string &sep = "\t\n ,;:",
							 const string &quote = "\"\'");

vector<string> breakup(const string &str,
					   const string &sep = "\t\n ,;:");

//
//	fullbreakup: break a string up into tokens separated by white-space
//			 characters - allowing for null size tokens
//
//
vector<string> fullbreakup( const string &str, const string &sep = ":" );


//
//	buildup: construct a string by concatenating the list of
//      separated tokens.  The separator is specified with sep.
//
string buildup( const vector<string>& tokens, char sep = ':' );


//
//	removeStringToken: Removes the specified token (the first
//                   token has position zero) token from the given
//                   list of separated tokens, and returns the result.
//                   The separator is given by sep.
//
string removeStringToken(
		const string&		tokens,
		unsigned int		position = 0,
		char			sep = ':' );


//
//	stringToken: Return the specified (the first token has position zero)
//                  token from the given list of separated tokens. 
//                  The separator is given by sep.
//
string stringToken(
		const string&		tokens,
		unsigned int		position = 0,
		char			sep = ':' );

//
//	isBlank: Returns true if the given string contains
//	only blank (ASCII 32) characters, or is empty.
//
bool isBlank(
		const string&		thisString );


#endif
