#include <stringutil.h>
#include <cassert>

int main()
{
	assert(trim("       ") == "");
	assert(trim("test ")			== "test" );
	assert(trim(" test ")			== " test" );
	assert(trim("")			== "" );
	assert(trim(" test ",true)            == "test");

	assert(unquote(" test ")		== " test " );
	assert(unquote("")			== "" );
	assert(unquote("'test'")		== "test" );
	assert(unquote("\"test\"")		== "test" );
	assert(unquote("'test")		== "'test" );
	assert(unquote("  'test'  ")		== "test" );
	assert(unquote("  ' test '   ")	== " test " );

	assert(lowercase(" HeLlO!1@ ")	== " hello!1@ " );

	assert(uppercase(" HeLlO!1@ ")	== " HELLO!1@ " );

	vector<string> v(breakup(",,,   : \t\t\t\n "));
	assert(!v.size());

	v = breakup("string");
	assert(v.size() == 1 && v[0]=="string");

	v = breakup("   ,,::x::y::|||12  ||2", " ,:|");
	assert(v.size()==4 && v[0]=="x" && v[1]=="y" && v[2]=="12" && v[3]=="2");

	v = quotedBreakup("", ":");
	assert(v.size()==0);

	v = quotedBreakup(":", ":");
	assert(v.size()==0);

	v = quotedBreakup("\"", ":");
	assert(v.size()==1 && v[0]=="\"");

	v = quotedBreakup("s1", ":");
	assert(v.size()==1 && v[0]=="s1");

	v = quotedBreakup("s1\"::\"x", ":");
	assert(v.size()==1 && v[0]=="s1\"::\"x");

	v = quotedBreakup("\"x::\":s1", ":");
	assert(v.size()==2 && v[0]=="\"x::\"" && v[1]=="s1");

	v = quotedBreakup("t=\"1 2\" x=1", " ");
	assert(v.size()==2 && v[0]=="t=\"1 2\"" && v[1]=="x=1");

	v = quotedBreakup("circle 348.5 359 44 text=\"yyy / zzz\" include=0", " ");
	assert(v.size()==6 && v[0]=="circle" && v[1]=="348.5" &&
		   v[2]=="359" && v[3]=="44" && v[4]=="text=\"yyy / zzz\"" &&
		   v[5]=="include=0");

	v = quotedBreakup("circle(1, 2, 3) text=\"yyy / zzz\" text={ok babe}",
					  " =", "\"({");
	assert(v.size()==5 && v[0]=="circle(1, 2, 3)" && v[1]=="text" &&
		   v[2]=="\"yyy / zzz\"" && v[3]=="text" && v[4]=="{ok babe}");

	assert(toNumber<int>("12334") == 12334 &&
		   toNumber<double>("3.1415") == 3.1415);

	return 0;
}
